from . import PetriNet
from . import Transition


class PetriNetEngine:

    def __init__(self, petri_net: PetriNet):
        self.petri_net = petri_net
        pass

    def search_transitions_enabled(self):
        return self.petri_net

    def fire_transition(self, transition: Transition):
        print("Firing transition")
        return transition



class Transition:

    def __init__(self, name: str, key: int):
        self.name: str = name
        self.id: int = key
        # self.outputs: list[int] = []

    # def is_transition_enable(self, places: list):
    #     validation = True
    #     for place in places:
    #         count = 0
    #         for input_value in place.outputs:
    #             if input_value == self.key:
    #                 count += 1
    #         validation = validation and place.validate_relations(count)
    #     return validation



class Place:

    def __init__(self, name: str, id: int, tokens: int):
        self.name: str = name
        self.id: int = id
        self.tokens: int = tokens

    def validate_relations(self, relations_number: int):
        return relations_number <= self.tokens



from . import Place
from . import Transition
import numpy as np


class ValidationPlaceExist(Exception):
    """Place exists"""
    pass


class ValidationTransitionExist(Exception):
    """Transition exists"""
    pass


class PetriNet:

    def __init__(self, places: list[Place], transitions: list[Transition], inputs: list[int][int],
                 outputs: list[int][int]):
        self.places: list[Place] = places
        self.transitions: list[Transition] = transitions
        self.inputs = np.array(inputs)
        self.outputs = np.array(outputs)

    def add_place(self, place_to_add: Place):
        for place in self.places:
            if place.id == place_to_add.id:
                return False
        self.places.append(place_to_add)
        return True

    def add_transition(self, transition_to_add: Transition):
        for transition in self.transitions:
            if transition.id == transition_to_add.id:
                # raise ValidationTransitionExist("Transicion ya existe")
                return False
        self.transitions.append(transition_to_add)
        return True

    def add_input(self, place: Place, transition: Transition):
        pass
